import {
  getLibp2p,
  getConnectedPeers,
  initIpfs,
  stopIpfs,
} from "@debros/network";
import { logger, startSpinner, stopSpinner } from "../utils/logger.js";
import { config } from "../utils/config.js";
import { NodeInfo } from "./ipfs.js";

// This service handles communication with remote DeBros network nodes
// It does not manage local node deployment, but rather allows the CLI
// to discover, communicate with, and deploy to remote nodes in the network

/**
 * Get all currently connected peers
 */
export async function getPeers(): Promise<any> {
  const spinner = startSpinner("Getting connected peers...");

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the connected peers
    const peers = getConnectedPeers();
    stopSpinner(`Found ${peers.length} connected peers`, true);

    return peers;
  } catch (error) {
    stopSpinner(
      `Failed to get peers: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    throw error;
  }
}

/**
 * Convert connected peers to NodeInfo format
 */
export async function getPeersAsNodeInfo(): Promise<NodeInfo[]> {
  const peers = await getPeers();
  const nodeInfos: NodeInfo[] = [];

  peers.forEach((data: any, peerId: any) => {
    const { load, publicAddress } = data;

    try {
      // Extract address and port from publicAddress
      const url = new URL(publicAddress);
      const address = url.hostname;
      const port = parseInt(url.port) || (url.protocol === "https:" ? 443 : 80);

      nodeInfos.push({
        id: peerId,
        address,
        port,
        load,
        type: "k3s", // Default to k3s, could be dynamically determined in the future
      });
    } catch (error) {
      logger.warn(
        `Invalid public address for peer ${peerId}: ${publicAddress}`
      );
    }
  });

  return nodeInfos;
}

/**
 * Announce a deployment to the network
 */
export async function announceDeployment(
  cid: string,
  appName: string,
  version: string = "latest",
  domain: string = ""
) {
  const spinner = startSpinner(
    `Announcing deployment of ${appName}:${version} to the network...`
  );

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Use default domain if not provided
    let appDomain = domain || `${appName}.${config.defaultDomain}`;

    // Ensure the domain ends with .debros.sol
    if (!appDomain.endsWith(`.${config.defaultDomain}`)) {
      // Remove any domain suffix if present
      const cleanSubdomain = appDomain.split(".")[0];
      appDomain = `${cleanSubdomain}.${config.defaultDomain}`;
    }

    // Create the deployment announcement message
    const message = {
      type: "deployment",
      cid,
      appName,
      version,
      domain: appDomain,
      timestamp: Date.now(),
    };

    // Publish to the debros-deploy topic
    await libp2p.services.pubsub.publish(
      "debros-deploy",
      new TextEncoder().encode(JSON.stringify(message))
    );

    stopSpinner(`Deployment announced to the network`, true);

    // Clean up IPFS
    await stopIpfs();

    return true;
  } catch (error) {
    stopSpinner(
      `Failed to announce deployment: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    throw error;
  }
}

/**
 * Listen for deployment announcements
 */
export async function listenForDeployments(
  callback: (message: {
    cid: string;
    appName: string;
    version: string;
    domain: string;
  }) => void
) {
  const spinner = startSpinner("Listening for deployment announcements...");

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Subscribe to the debros-deploy topic
    await libp2p.services.pubsub.subscribe("debros-deploy");

    // Set up the message handler
    libp2p.services.pubsub.addEventListener("message", (event: any) => {
      try {
        if (event.detail.topic === "debros-deploy") {
          const msgData = new TextDecoder().decode(event.detail.data);
          const message = JSON.parse(msgData);

          if (message.type === "deployment") {
            callback({
              cid: message.cid,
              appName: message.appName,
              version: message.version,
              domain:
                message.domain || `${message.appName}.${config.defaultDomain}`,
            });
          }
        }
      } catch (error) {
        logger.error(
          `Failed to process deployment message: ${
            error instanceof Error ? error.message : String(error)
          }`
        );
      }
    });

    stopSpinner("Listening for deployment announcements", true);
    return true;
  } catch (error) {
    stopSpinner(
      `Failed to listen for deployments: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    throw error;
  }
}

/**
 * Get versions of an application
 */
export async function getAppVersions(appName: string): Promise<
  Array<{
    tag: string;
    cid: string;
    timestamp: string;
    deployed: boolean;
  }>
> {
  const spinner = startSpinner(`Fetching versions for ${appName}...`);

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Get application versions from network
    // In a real implementation, this would query OrbitDB or another storage mechanism

    // Simulate fetching version info from network
    // This would typically come from querying the network or local database
    const versions = [
      {
        tag: "latest",
        cid: "QmVersion1",
        timestamp: new Date().toISOString(),
        deployed: true,
      },
      {
        tag: "v1.0.0",
        cid: "QmVersion2",
        timestamp: new Date(Date.now() - 24 * 60 * 60 * 1000).toISOString(), // 1 day ago
        deployed: false,
      },
      {
        tag: "v0.9.0",
        cid: "QmVersion3",
        timestamp: new Date(Date.now() - 7 * 24 * 60 * 60 * 1000).toISOString(), // 1 week ago
        deployed: false,
      },
    ];

    stopSpinner(`Found ${versions.length} versions for ${appName}`, true);

    // Clean up IPFS
    await stopIpfs();

    return versions;
  } catch (error) {
    stopSpinner(
      `Failed to fetch versions: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    throw error;
  }
}

/**
 * Rollback an application to a specific version
 */
export async function rollbackToVersion(
  appName: string,
  versionTag?: string,
  domain?: string,
  force: boolean = false
): Promise<{
  success: boolean;
  message?: string;
  version?: string;
  cid?: string;
}> {
  const spinner = startSpinner(
    `Rolling back ${appName} to ${versionTag || "previous version"}...`
  );

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Get versions to find the right one
    const versions = await getAppVersions(appName);

    if (versions.length === 0) {
      stopSpinner(`No versions found for ${appName}`, false);
      return { success: false, message: `No versions found for ${appName}` };
    }

    // Find the requested version or previous version
    let targetVersion;

    if (versionTag) {
      // Find specific version
      targetVersion = versions.find((v) => v.tag === versionTag);
      if (!targetVersion) {
        stopSpinner(`Version ${versionTag} not found for ${appName}`, false);
        return { success: false, message: `Version ${versionTag} not found` };
      }
    } else {
      // Find previous version (not current)
      const currentVersion = versions.find((v) => v.deployed);
      const otherVersions = versions
        .filter((v) => !v.deployed)
        .sort(
          (a, b) =>
            new Date(b.timestamp).getTime() - new Date(a.timestamp).getTime()
        );

      if (otherVersions.length === 0) {
        stopSpinner(`No previous versions found for ${appName}`, false);
        return {
          success: false,
          message: "No previous versions found to rollback to",
        };
      }

      targetVersion = otherVersions[0];
    }

    // Announce the rollback deployment with the CID of the target version
    const appDomain = domain || `${appName}.${config.defaultDomain}`;

    // Create the deployment announcement message
    const message = {
      type: "deployment",
      cid: targetVersion.cid,
      appName,
      version: targetVersion.tag,
      domain: appDomain,
      isRollback: true,
      timestamp: Date.now(),
    };

    // Publish to the debros-deploy topic
    await libp2p.services.pubsub.publish(
      "debros-deploy",
      new TextEncoder().encode(JSON.stringify(message))
    );

    stopSpinner(`Rolled back ${appName} to version ${targetVersion.tag}`, true);

    // Clean up IPFS
    await stopIpfs();

    return {
      success: true,
      version: targetVersion.tag,
      cid: targetVersion.cid,
    };
  } catch (error) {
    stopSpinner(
      `Failed to rollback: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    return {
      success: false,
      message: error instanceof Error ? error.message : String(error),
    };
  }
}

/**
 * Stop an application
 */
export async function stopApplication(
  appName: string,
  force: boolean = false
): Promise<{
  success: boolean;
  message?: string;
}> {
  const spinner = startSpinner(`Stopping application ${appName}...`);

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Create the stop message
    const message = {
      type: "application-action",
      action: "stop",
      appName,
      force,
      timestamp: Date.now(),
    };

    // Publish to the debros-app-actions topic
    await libp2p.services.pubsub.publish(
      "debros-app-actions",
      new TextEncoder().encode(JSON.stringify(message))
    );

    stopSpinner(
      `Stop command for ${appName} has been sent to the network`,
      true
    );

    // Clean up IPFS
    await stopIpfs();

    return { success: true };
  } catch (error) {
    stopSpinner(
      `Failed to stop application: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    return {
      success: false,
      message: error instanceof Error ? error.message : String(error),
    };
  }
}

/**
 * Start an application
 */
export async function startApplication(
  appName: string,
  versionTag: string = "latest"
): Promise<{
  success: boolean;
  message?: string;
}> {
  const spinner = startSpinner(
    `Starting application ${appName} with version ${versionTag}...`
  );

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Get versions to find the right CID
    const versions = await getAppVersions(appName);

    if (versions.length === 0) {
      stopSpinner(`No versions found for ${appName}`, false);
      return { success: false, message: `No versions found for ${appName}` };
    }

    // Find the requested version
    const targetVersion = versions.find((v) => v.tag === versionTag);
    if (!targetVersion) {
      stopSpinner(`Version ${versionTag} not found for ${appName}`, false);
      return { success: false, message: `Version ${versionTag} not found` };
    }

    // Create the start message
    const message = {
      type: "application-action",
      action: "start",
      appName,
      version: versionTag,
      cid: targetVersion.cid,
      timestamp: Date.now(),
    };

    // Publish to the debros-app-actions topic
    await libp2p.services.pubsub.publish(
      "debros-app-actions",
      new TextEncoder().encode(JSON.stringify(message))
    );

    stopSpinner(
      `Start command for ${appName} with version ${versionTag} has been sent to the network`,
      true
    );

    // Clean up IPFS
    await stopIpfs();

    return { success: true };
  } catch (error) {
    stopSpinner(
      `Failed to start application: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    return {
      success: false,
      message: error instanceof Error ? error.message : String(error),
    };
  }
}

/**
 * Delete an application
 */
export async function deleteApplication(
  appName: string,
  force: boolean = false
): Promise<{
  success: boolean;
  message?: string;
}> {
  const spinner = startSpinner(`Deleting application ${appName}...`);

  try {
    // Initialize IPFS if it's not already running
    await initIpfs();

    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      throw new Error("LibP2P is not initialized");
    }

    // Create the delete message
    const message = {
      type: "application-action",
      action: "delete",
      appName,
      force,
      timestamp: Date.now(),
    };

    // Publish to the debros-app-actions topic
    await libp2p.services.pubsub.publish(
      "debros-app-actions",
      new TextEncoder().encode(JSON.stringify(message))
    );

    stopSpinner(
      `Delete command for ${appName} has been sent to the network`,
      true
    );

    // Clean up IPFS
    await stopIpfs();

    return { success: true };
  } catch (error) {
    stopSpinner(
      `Failed to delete application: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    return {
      success: false,
      message: error instanceof Error ? error.message : String(error),
    };
  }
}

/**
 * Stop listening for deployment announcements
 */
export async function stopListeningForDeployments() {
  const spinner = startSpinner("Stopping deployment listener...");

  try {
    // Get the libp2p instance
    const libp2p = getLibp2p();
    if (!libp2p) {
      spinner.text = "LibP2P is not initialized, nothing to stop";
      stopSpinner("Stopped deployment listener", true);
      return true;
    }

    // Unsubscribe from the debros-deploy topic
    await libp2p.services.pubsub.unsubscribe("debros-deploy");

    // Stop IPFS
    await stopIpfs();

    stopSpinner("Stopped deployment listener", true);
    return true;
  } catch (error) {
    stopSpinner(
      `Failed to stop deployment listener: ${
        error instanceof Error ? error.message : String(error)
      }`,
      false
    );
    throw error;
  }
}
